#
# See the file "LICENSE" for the full license governing this code.
#
#
# This class takes care of installing the IIB software
#
define iib_install::software(
  String                               $source_location,
  String                               $install_root    = '/opt/IBM',      # Root file where installtion is places
  String                               $tmp_dir         = '/tmp',          # Directory used for unpacking
  Variant[Enum['on_failure'], Boolean] $logoutput       = 'on_failure',    # Output of the execs
)
{
  if $::osfamily == 'windows' { fail 'This module does not support Windows'}
  iib_install { "iib_install::software::${title}":}

  $version = $title
  assert_type(Pattern[/\d+\.\d+\.\d+\.\d/], $version) |$expected, $actual| {
    fail 'Invalid version specified'
  }

  validate_absolute_path($install_root)

  assert_type(Pattern[/\/opt\/.*/], $install_root) |$expected, $actual| {
    fail "install_root needs to be a sub directory of /opt, right now it isn't"
  }

  #
  # Setup some variables
  #
  $supported_major_versions = ['10']    # At this point in time only version 10 is tested. Probably other versions work as well.
  $supported_patch_levels   = range(6,11)
  $version_components = $version.split('\.')
  #
  # On the package version the last number is separated by an dash
  #
  $base_version    = "${version_components[0]}.${version_components[1]}.${version_components[2]}"
  $major_version   = $version_components[0]

  #
  # This only works on higher versions of Puppet
  # TODO: Use this when support for Puppet 4.3 and lower is removed.
  # $patch_level     = Integer($version_components[3])
  #
  $patch_level       = $version_components[3].scanf('%i')[0]
  #
  # This only works on higher versions of Puppet
  # TODO: Use this when support for Puppet 4.3 and lower is removed.
  # $file_name       = "${base_version}-IIB-LINUXX64-FP${String($patch_level,'%04d')}"
  #
  $file_name       = "${base_version}-IIB-LINUXX64-FP${sprintf('%04d', $patch_level)}"
  $dirs            = dirtree($install_root, '/opt')

  unless $major_version in $supported_major_versions {
    fail "IIB major version ${major_version} not (yet) supported. Only versions ${supported_major_versions.join(',')} supported."
  }

  unless $patch_level in $supported_patch_levels {
    fail "IIB patch level ${patch_level} not (yet) supported. Only patchlevels ${supported_patch_levels.join(',')} supported."
  }

  #
  # If the installed version is the same as the requested version. Do nothing
  #
  if $::iib_versions == undef or $version != $::iib_versions["${install_root}/iib-${version}"] {

    case $patch_level {
      6,7: {
        $software_file = "${tmp_dir}/EAsmbl_image/iib-${version}.tar.gz"
        iib_install::software::iib6_unpack{ $file_name:
          tmp_dir         => $tmp_dir,
          source_location => $source_location,
          before          => Archive[$archive_title],
        }
      }
      default: {
        $software_file = "${source_location}/${file_name}.tar.gz"
        $archive_title = "${install_root}/${file_name}.tar.gz"
      }
    }

    $dirs.each |$dir| {
      unless defined(File[$dir]) {
        file {$dir:
          ensure => 'directory',
          owner  => 'root',
          mode   => '0644',
          before => Archive[$archive_title],
        }
      }
    }

    archive { $archive_title:
      ensure       => present,
      extract      => true,
      extract_path => $install_root,
      source       => $software_file,
      creates      => "${install_root}/iib-${version}",
      cleanup      => true,
    }

    ~> exec { "Accept IIB License Requirements for version ${version}":
      command     => "${install_root}/iib-${version}/iib make registry global accept license silently",
      refreshonly => true,
      logoutput   => $logoutput,
    }

    -> exec { "Install IIB version ${version}":
      cwd       => "${install_root}/iib-${version}",
      command   => "${install_root}/iib-${version}/iib verify all",
      logoutput => $logoutput,
    }
  }
}
